#include "Arduino.h"
#include "Wire.h"
#include "AQM1602Y.h"

#define CMD_MODE 0x00
#define DATA_MODE 0x40

AQM1602Y::AQM1602Y( int addr, int volt ){
    i2caddr = addr;
    if ( volt == 5 ){
        vdd = 5;
    }else{
        vdd =3;
    }
}

void AQM1602Y::begin(){
    cursol = 1;
    blink = 1;
    display = 1;
    x = 0;
    y = 0;

    byte br, bon;

    delay( 145 );
    lcd_write( CMD_MODE, 0x38 );
    delay( 5 );
    lcd_write( CMD_MODE, 0x38 );
    delay( 5 );

    if ( i2caddr = 0x3E ){
        if ( vdd == 5 ){
            br = 0x23;
            bon = 0x00;
        }else{
            br = 0x13;
            bon = 0x04;
        }
        lcd_write( CMD_MODE, 0x39 );
        delay( 5 );
        lcd_write( CMD_MODE, 0x14 );
        delay( 5 );
        lcd_write( CMD_MODE, 0x70 | ( br & 0x0F ) );
        delay( 5 );
        lcd_write( CMD_MODE, 0x50 | ( bon | ( br >> 4) ) );
        delay( 5 );
        lcd_write( CMD_MODE, 0x6C );
        delay( 5 );
        lcd_write( CMD_MODE, 0x38 );
        delay( 5 );       
    }

    lcd_write( CMD_MODE, 0x01 );
    delay( 5 );
    lcd_write( CMD_MODE, 0x0C );
    delay( 5 );

}

void AQM1602Y::clear() {
    lcd_write( CMD_MODE, 0x01 );
    delay( 10 );
}

void AQM1602Y::set_cursol(int flag) {
    if ( flag != 0 ) {
        flag = 1;
    }
    cursol = flag;
    set_display();
}

void AQM1602Y::set_blink(int flag) {
    if ( flag != 0 ) {
        flag = 1;
    }
    blink = flag;
    set_display();
}

void AQM1602Y::set_contrast( int value ) {
    if ( value < 0x00 ) {
        value = 0x00;
    }
    if ( value > 0x0f ) {
        value = 0x0f;
    }
    contrast = value;
    lcd_write( CMD_MODE, 0x70 | contrast );
}

void AQM1602Y::set_display() {
    int buf;
    buf = 0x08 + 0x04 * display + 0x02 * cursol + blink;
    lcd_write( CMD_MODE, buf );
    delay( 10 );
}

void AQM1602Y::move(int mx, int my) {
    byte buf;
    x = mx;
    y = my;
    if ( x < 0 ){
        x = 0x00;
    }
    if ( x > 0x0f){
        x = 0x0f;
    }
    if ( y < 0 ){
        y = 0x00;
    }
    if ( y > 0x01){
        y = 0x01;
    }
    
    buf = x + ( y * 0x40 ) + 0x80;
    lcd_write( CMD_MODE, buf );
    delay( 10 );
}

void AQM1602Y::charwrite(char *outchar) {
    String str;
    str = outchar;
    int i,length;
    
    length = str.length();
    for ( i = 0; i < length; i++){
        if ( x > 0x0f ){
            if ( y == 0 ){
                move( 0x00, 0x01 );
                y = 0x01;
            }else{
                break;
            }
        }
        lcd_write( DATA_MODE, str.charAt(i) );
        x++;
    }

}

void AQM1602Y::lcd_write( int mode, int cmd ) {
    Wire.beginTransmission( i2caddr );
    Wire.write( mode );
    Wire.write( cmd );
    Wire.endTransmission();    
}
